#!/usr/bin/env python
# encoding: utf-8

import logging
import os
import sys

from cspp_some_utils.log_common import configure_logging, status_line

from adl_common import PACKAGE_DATABASE, CACHE_DATABASE
from cspp_some_utils.basics import check_existing_env_var
from cspp_some_utils.basics import check_and_convert_path
from adl_common import get_cspp_static_ancil_path, get_and_check_cspp_dynamic_ancil_dir
import traceback
import datetime as dt
import sqlite3
import re
import time
import shutil
import glob
#import stat 
from stat import *
from adl_common import get_shipped_db, get_installed_db
import tempfile
import pprint as pp


LOG = logging.getLogger(__name__)

# use this set of required keys when scanning RDR granules with skim_dir
RDR_REQUIRED_KEYS = [
    'N_Granule_ID', 'n_collection_short_name', 'StartTime', 'EndTime']
POLARWANDER_REQUIRED_KEYS = [
    'EffectiveStartTime', 'EffectiveEndTime', 'n_collection_short_name', 'N_Software_Version']

SOFT_VERSION_KEY = 'N_Software_Version'

# 'COLLECTION_SHORT_NAME',

NEEDED_PROPS = ['GRANULE_ID', 'RANGE_START_IET', 'RANGE_STOP_IET', 'EFFECTIVITY_START_IET',
                'EFFECTIVITY_STOP_IET', 'PRODUCT_ID', 'GRANULE_VERSION']

NEEDED_PROPS_DT = ['RANGE_START_IET', 'RANGE_STOP_IET', 'EFFECTIVITY_START_IET',
                   'EFFECTIVITY_STOP_IET']

NEEDED_SHORT_NAMES = ['VIIRS-SCIENCE-RDR', 'CRIS-SCIENCE-RDR', 'ATMS-SCIENCE-RDR',
                      'SPACECRAFT-DIARY-RDR',
                      'VIIRS-RSBAUTOCAL-HISTORY-AUX', 'TLE-AUX-Int']

COMPAT = {'RANGE_START_IET': 'StartTime', 'EFFECTIVITY_START_IET': 'StartTime', 'GRANULE_VERSION':
          'N_Granule_Version','GRANULE_ID': 'N_Granule_ID'}

global_database_name = None
sud_dir_list = ['AUX', 'ANC']

#PACKAGE_DATABASE = 'cspp_sdr_package.db'
#CACHE_DATABASE = 'cspp_sdr_cache.db'

WORK_DATABASE = 'cspp_sdr_work.db'
SHIPPED_LUTS = 'shipped_luts'
STRAY_LUTS = 'stray_light_luts'
INCOMING_LUTS = 'incoming_luts'




    
def shipped_luts_abs_path():
    """
    """
 
    static_path = get_cspp_static_ancil_path()
    LOG.debug('static_path %s'%static_path)

    return os.path.join(static_path, SHIPPED_LUTS)

def stray_light_abs_path():
    """
    """    

 
    static_path = get_cspp_static_ancil_path()
    return os.path.join(static_path, STRAY_LUTS)


def incoming_luts_abs_path():
    """
    """
    return os.path.join(get_and_check_cspp_dynamic_ancil_dir(), INCOMING_LUTS)


def replace_file_with_link(cached_file, file_to_link):
    """
    """
    if not os.path.exists(cached_file):
        LOG.error("No cached file %s" % cached_file)
        return

    LOG.debug('SOFT LINK src %s dest %s' % (cached_file, file_to_link))
    if not os.path.islink(cached_file):
        tname=file_to_link+".t"
        os.symlink(cached_file, tname)
        os.rename(tname,file_to_link)
        
    if not os.path.islink(file_to_link):
        LOG.error('link failed %s'%file_to_link)

def replace_file_with_stub(cached_file, file_to_link):
    """
    """
    if not os.path.exists(cached_file):
        LOG.error("No cached file %s" % cached_file)
        return

    LOG.debug('FILE STUB src %s dest %s' % (cached_file, file_to_link))
    if not os.path.islink(cached_file):
        stub_file=open(file_to_link, "w+")

        stub_file.write("fakedata") 
        #stub_file.truncate()
        stub_file.close()
        
    if not os.path.isfile(file_to_link):
        LOG.error('STUB failed %s'%file_to_link)


def count_soft_links(indir):
    """
    """
    links = 0
    for root, dirs, files in os.walk(indir):
        for f in files:
            #LOG.info('file %s'%(f))
            if  os.path.islink(os.path.join(root,f)):
                    links += 1

    #LOG.info('lnks %s'% str(links))   
    
    if links == 0 :
        LOG.warning('\n   (Installation may be incomplete! Please run sdr_luts.sh)\n')        
    return links
    
def create_lut_soft_links(working_dms_path, lut_cache):
    """
    """
    if os.path.isfile(working_dms_path) :
        db_files = os.path.dirname(working_dms_path)
    else :
        db_files = working_dms_path

    for root, dirs, files in os.walk(db_files):
        for f in files:
            lookfor = ".".join(os.path.basename(f).split(".")[1:])

            real_file = glob.glob(os.path.join(lut_cache, lookfor))
            #LOG.info('file %s %d'%(lookfor,len(real_file)))
            if len(real_file) == 1:
                LOG.debug('found %s %s' % (f,real_file[0]))
                replace_file_with_link(real_file[0], os.path.join(root, f))
                
def _copytree(src, dst, symlinks=False):
    names = os.listdir(src)
    os.makedirs(dst)
    errors = []
    for name in names:
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        try:
            if symlinks and os.path.islink(srcname):
                linkto = os.readlink(srcname)
                os.symlink(linkto, dstname)
            elif os.path.isdir(srcname):
                _copytree(srcname, dstname, symlinks)
            else:
                shutil.copy2(srcname, dstname)
            # XXX What about devices, sockets etc.?
        except OSError as why:
            errors.append((srcname, dstname, str(why)))
        # catch the Error from the recursive copytree so that we can
        # continue with other files
        except Exception as err:
            errors.extend(err.args[0])
    try:
        shutil.copystat(src, dst)
    except OSError as why:
        # can't copy file access times on Windows
            errors.extend((src, dst, str(why)))
    if errors:
        raise Exception(errors)


def create_cache_database(cspp_sdr_home, overwrite=True):
    """
    """
    LOG.info('create_cache_database %s'%cspp_sdr_home)
    #cache_dms_path =  os.path.dirname(get_installed_db(cspp_sdr_home) )

    overwrite=True
        # copy the PACKAGE_DATABASE to the daily_location
    global global_database_name
    
    package_dms_path =  os.path.dirname(get_shipped_db(cspp_sdr_home) )
    #luts_cache = os.path.join(get_and_check_cspp_dynamic_ancil_dir(), INCOMING_LUTS)
    
        
    installed_dms_path =  os.path.dirname(get_installed_db(cspp_sdr_home) )
    cache_dms_path = installed_dms_path
    
    cache_dir=get_and_check_cspp_dynamic_ancil_dir()
    tf= tempfile.TemporaryDirectory( dir=cache_dir)
    tmp_dms_path = os.path.join(tf.name,"tmp")

    
    LOG.debug('package_dms_path %s'%package_dms_path)
    LOG.debug('tmp_dms_path %s'%tmp_dms_path)
    LOG.debug('cache_dms_path %s'%cache_dms_path)
    
    if not os.path.exists(cache_dms_path):
        overwrite = True
    
    if overwrite is True:
        status_line('One time initialization, takes some time')
    
        #shutil.rmtree(tmp_dms_path, ignore_errors=True)
            #shutil.rmtree(cache_dms_path, ignore_errors=True)

    
        startTime = time.time()
    
        LOG.debug('copy %s to \n %s' % (package_dms_path, tmp_dms_path))
    
        dms_dest_db = os.path.join(tmp_dms_path, CACHE_DATABASE)
    
        tmp_db = os.path.join(tmp_dms_path, PACKAGE_DATABASE)
         

        st_src = os.stat(package_dms_path)
        LOG.info("st_src %s"%str(st_src))
        _copytree(package_dms_path, tmp_dms_path)
            #copy_tree(package_dms_path, tmp_dms_path)
         
        os.rename(tmp_db,dms_dest_db)
    
        endTime = time.time()
        delta = endTime - startTime
    
        status_line('Execution Time: %f Sec ' % delta)
        os.rename(tmp_dms_path, cache_dms_path)
 
    dms_dest_db = os.path.join(cache_dms_path, CACHE_DATABASE)
    global_database_name = "file:" + dms_dest_db + ":?cache=shared"
    global_database_name = dms_dest_db

    LOG.debug('create_cache_database - end')

    return dms_dest_db


def is_cache_initialized(cspp_sdr_home):
    """
    """

    cache_dms_path =  os.path.dirname(get_installed_db(cspp_sdr_home) )
    return os.path.exists(cache_dms_path)


def setup_dms_in_work_directory(work_dir, cspp_sdr_home):
    """
    Copy the dms and make needed links in work directory
    :return:
    """
    global global_database_name


    cache_dms_path =  os.path.dirname(get_installed_db(cspp_sdr_home) )   
    #create_cache_database(cspp_sdr_home)

    working_dms_path = os.path.join(work_dir, 'db')
    dms_dest_db = os.path.join(working_dms_path, WORK_DATABASE)
    global_database_name = dms_dest_db
    
    LOG.info('setup_dms_in_work_directory cache %s work %s'%(cache_dms_path,working_dms_path))
    
    try :
        os.makedirs(working_dms_path)

        shutil.copyfile(os.path.join(cache_dms_path, CACHE_DATABASE) , os.path.join(working_dms_path, WORK_DATABASE))
        
        # SRM test
        st_src = os.stat(cache_dms_path)
        LOG.info("st_src %s"%str(st_src))

        shutil.copytree(os.path.join(cache_dms_path, 'DmSm') ,os.path.join(working_dms_path, 'DmSm'))

        luts_cache = stray_light_abs_path()
        create_lut_soft_links(working_dms_path, luts_cache)
             
        luts_cache = shipped_luts_abs_path()
        create_lut_soft_links(working_dms_path, luts_cache)
  
        luts_cache = incoming_luts_abs_path()
        create_lut_soft_links(working_dms_path, luts_cache)
             

        perm= S_IRUSR | S_IWUSR | S_IXUSR
           
        for root, dirs, files in os.walk(os.path.join(working_dms_path, 'db'), followlinks=False):
                for d in dirs:
                    os.chmod(os.path.join(root, d), perm)
                for f in files:
                    os.chmod(os.path.join(root, f), perm)
         

        LOG.debug('global name %s'%global_database_name)
        conn = sqlite3.connect(global_database_name)
        cursor = conn.cursor()
    
        table = 'TDMS_COREFL_CORE_FILES'
        myproperty = 'DEVICE_NAME'
    
        static_path = get_cspp_static_ancil_path()
        dms_tiles = os.path.join(static_path, 'ecotiles')
        old_value = '%anc/static/ecotiles%'
     
        cmd = 'UPDATE %s SET %s = "%s" WHERE %s LIKE "%s"' % (
            table, myproperty, dms_tiles, myproperty, old_value)
        LOG.debug(cmd)
        cursor.execute(cmd)
        conn.commit()
        conn.close()



        os.makedirs(os.path.join(working_dms_path,"fin"))    
    except :
        #logging.error(traceback.format_exc())
        #LOG.error("setup stuff - continuing - remove this after testing")
          
        wait_cnt=0
        while not os.path.exists( os.path.join(working_dms_path,"fin") ):
            # lazy lock for above critical section
            wait_cnt += 1
            if wait_cnt > 600 :
                break;
            LOG.info('wait for working db initialization %s'%str(wait_cnt))
            time.sleep( 5 )
            
        LOG.warning('DMS copy skipped %s already exists, Reusing work directory.' % working_dms_path)



    count_soft_links(working_dms_path)

    return global_database_name


def update_orbit_number(orbit_number, granule_id):
    """
    """

    g_id = 'GRANULE_ID'
    'RANGE_START_IET'

    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()

    table = 'TDMS_COREUR_CORE_URS'
    
    myproperty = 'BEGIN_ORBIT_NUMBER'

    cmd = 'UPDATE %s SET %s = "%s" WHERE %s LIKE "%s"' % (
        table, myproperty, orbit_number, g_id, granule_id)
    LOG.debug(cmd)

    cursor.execute(cmd)
    conn.commit()
    conn.close()

def is_file_in_database(filename,dbname=None):
    """
    SELECT * FROM TDMS_COREFL_CORE_FILES WHERE FILE_NAME LIKE 'CrIS-Correct-Matrix-AUX_j01_20180814183415Z_20191100174748Z_ee00000000000000Z_-_nobc_ops_all-_ops.h5';
    """
    in_database=False
    #if global_database_name is not None:
    use_db=global_database_name
    if dbname is not None:
        use_db=dbname
        
    conn = sqlite3.connect(use_db)
    cursor = conn.cursor()
    
    table = 'TDMS_COREFL_CORE_FILES'
     
    myproperty = 'FILE_NAME'
    
    cmd = 'SELECT %s FROM %s WHERE %s LIKE "%s"' % (
          myproperty,table, myproperty,filename)
    LOG.debug(cmd)
    
    cursor.execute(cmd)
    device=cursor.fetchall()

    conn.commit()
    conn.close() 
    if len( device ) > 0 :
        in_database = True
 
    return in_database
    
def check_science_RDR(spacecraft='NPP'):
    """
    """
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()


    cmd = 'SELECT  COREUR_ID FROM TDMS_COREMD_METADATA where STRING_VALUE LIKE "%SPACECRAFT"'
    LOG.debug(cmd)
    cursor.execute(cmd)

    core_urids=cursor.fetchall()
    LOG.debug(core_urids)

    conn.commit()
    conn.close()
    
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()

    work_dms_files = os.path.join(os.path.dirname(global_database_name),'DmSm')
    
    all_rdr_sizes=[]

    for coreur_id in core_urids:
    
        cmd = 'SELECT  DEVICE_NAME FROM TDMS_COREFL_CORE_FILES where COREUR_ID LIKE "%s"'%coreur_id
        LOG.debug(cmd)
        cursor.execute(cmd)
        device=cursor.fetchall()
        filepath=os.path.join(work_dms_files,device[0][0],"*.CRIS-SCIENCE-RDR")

        filenames=glob.glob(filepath)
 
        for filename in filenames:
            if os.path.isfile(filename) :
                npp_sizes=[7300000,9390240,7313248,7260174] #, 4447728, 4455458]  
                jpss_sizes=[9390240,9397970] 
                if spacecraft=='NPP' :
                    sizes = npp_sizes
                else :
                    sizes = jpss_sizes
               
                fsize = int(os.path.getsize(filename))
                
                all_rdr_sizes.append(fsize)
                
                ok = False
                for gsize in sizes:
                    LOG.debug('%s %d %d'%(filename,os.path.getsize(filename),gsize))
                    if (gsize <=  fsize)   :
                        ok =True
                        
                if not ok:
                    LOG.info('Removed CRIS-SCIENCE-RDR %s %s %s'%(spacecraft,filename,str(os.path.getsize(filename))))
                    smallfile=filename+".small"
                    os.rename(filename,smallfile)
                    urid=os.path.basename(filename).split(".")[0]
                    cmd = 'DELETE FROM TDMS_COREUR_CORE_URS where URID LIKE "%s" '%urid
                    LOG.debug(cmd)
                    cursor.execute(cmd)
                    #result=cursor.fetchall()
             
    conn.commit()
    conn.close()
    
    all_rdr_sizes=sorted(set(all_rdr_sizes))
    LOG.info("%s RDR sizes %s"%(spacecraft,pp.pformat(all_rdr_sizes)))
    
def check_atms_science_RDR(spacecraft='NPP'):
    """
    """
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()


    cmd = 'SELECT  COREUR_ID FROM TDMS_COREMD_METADATA where STRING_VALUE LIKE "%SPACECRAFT"'
    LOG.debug(cmd)
    cursor.execute(cmd)

    core_urids=cursor.fetchall()
    LOG.debug(core_urids)

    conn.commit()
    conn.close()
    
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()

    work_dms_files = os.path.join(os.path.dirname(global_database_name),'DmSm')

    all_rdr_sizes=[]

    for coreur_id in core_urids:
    
        cmd = 'SELECT  DEVICE_NAME FROM TDMS_COREFL_CORE_FILES where COREUR_ID LIKE "%s"'%coreur_id
        LOG.debug(cmd)
        cursor.execute(cmd)
        device=cursor.fetchall()

        filepath=os.path.join(work_dms_files,device[0][0],"*.ATMS-SCIENCE-RDR")
        filenames=glob.glob(filepath)
 
        for filename in filenames:
            if os.path.isfile(filename) :
 
                npp_sizes= [111102,111042]  # ,9375552,9383282
                jpss_sizes=[111102,111042] 
 
                if spacecraft=='NPP' :
                    sizes = npp_sizes
                else :
                    sizes = jpss_sizes
               
                fsize = int(os.path.getsize(filename))
                
                all_rdr_sizes.append(fsize)
                
                ok = False
                for gsize in sizes:
                    LOG.debug('%s %d %d'%(filename,os.path.getsize(filename),gsize))
                    if (gsize <=  fsize)   :
                        ok =True
                        
                if not ok:
                    LOG.info('Removed ATMS-SCIENCE-RDR %s %s %s'%(spacecraft,filename,str(os.path.getsize(filename))))
                    smallfile=filename+".small"
                    os.rename(filename,smallfile)
                    urid=os.path.basename(filename).split(".")[0]
                    cmd = 'DELETE FROM TDMS_COREUR_CORE_URS where URID LIKE "%s" '%urid
                    LOG.debug(cmd)
                    cursor.execute(cmd)
                    #result=cursor.fetchall()
             
    conn.commit()
    conn.close()    
    
    
    all_rdr_sizes=sorted(set(all_rdr_sizes))
    LOG.info("%s RDR sizes %s"%(spacecraft,pp.pformat(all_rdr_sizes)))
    
    
def check_spacecraft_diaries(spacecraft='NPP'):
    """
    """
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()


    cmd = 'SELECT  COREUR_ID FROM TDMS_COREMD_METADATA where STRING_VALUE LIKE "%SPACECRAFT"'
    LOG.debug(cmd)
    cursor.execute(cmd)

    core_urids=cursor.fetchall()
    LOG.debug(core_urids)

    conn.commit()
    conn.close()
    
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()

    work_dms_files = os.path.join(os.path.dirname(global_database_name),'DmSm')

    for coreur_id in core_urids:
    
        cmd = 'SELECT  DEVICE_NAME FROM TDMS_COREFL_CORE_FILES where COREUR_ID LIKE "%s"'%coreur_id
        LOG.debug(cmd)
        cursor.execute(cmd)
        device=cursor.fetchall()
        filepath=os.path.join(work_dms_files,device[0][0],"*.SPACECRAFT-DIARY-RDR")
        filenames=glob.glob(filepath)
 
        for filename in filenames:
            #LOG.debug('%s %d '%(filename,os.path.getsize(filename)))

            if os.path.getsize(filename) not in {14340, 14437, 14620, 14803}:
                LOG.info('Removed SPACECRAFT-DIARY-RDR %s %s'%(filename,str(os.path.getsize(filename))))
                smallfile=filename+".small"
                os.rename(filename,smallfile)
                urid=os.path.basename(filename).split(".")[0]
                cmd = 'DELETE FROM TDMS_COREUR_CORE_URS where URID LIKE "%s" '%urid
                LOG.debug(cmd)
                cursor.execute(cmd)
                #result=cursor.fetchall()

    conn.commit()
    conn.close()
   
def get_product_ids(spacecraft='NPP'):
    """
    """
    LOG.debug('Open %s' % global_database_name)
    property_lookup = {}
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()

    cursor.execute('SELECT name FROM sqlite_master WHERE type=\'table\';')
    tables = cursor.fetchall()
    LOG.debug(tables)

    for table_info in tables:

        info_str = str(table_info)
        table_name = re.search('[a-zA-Z0-9_]+', info_str)
        table_name = info_str[table_name.start():table_name.end()]
        # LOG.info('Table %s' % (table_name))
        # Retrieve column information
        # Every column will be represented by a tuple with the following attributes:
        # (id, name, type, notnull, default_value, primary_key)
        cursor.execute('PRAGMA TABLE_INFO({})'.format(table_name))

        # collect names in a list
        names = [tup[1] for tup in cursor.fetchall()]
        for name in names:
            props = NEEDED_PROPS
            # props.append('COLLECTION_SHORT_NAME')
            if name in props + ['COLLECTION_SHORT_NAME']:
                LOG.debug('%s Property %s' % (table_name, name))
                property_lookup[name] = table_name

    product_id = {}

    # now find the product ids and shortname
    # find the product ids
    product_table = property_lookup['COLLECTION_SHORT_NAME']
    for short_name in NEEDED_SHORT_NAMES:
        query = (
            'SELECT * FROM {0:s} WHERE {1:s} LIKE "{2:s}" AND PLATFORM_SHORT_NAME like "{3:s}" '.format(product_table,
                                                                                                        'COLLECTION_SHORT_NAME',
                                                                                                        short_name,
                                                                                                        spacecraft))
        LOG.debug(query)
        cursor.execute(query)
        # cursor.execute('SELECT * FROM {tn} WHERE {cn} LIKE "{cv}"'.\
        # format(tn=product_table, cn='COLLECTION_SHORT_NAME', cv=short_name))

        all_rows = cursor.fetchall()
        for row in all_rows:
            product_id[row[1]] = row[0]

    conn.close()

    return product_id, property_lookup


def lookup_by_product_id_and_replace_value(myproperty, new_value, old_value):
    """
    :param product_id:
    :return:
    """
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()

    table = 'TDMS_COREUR_CORE_URS'

    cmd = 'UPDATE %s SET %s = "%s" WHERE %s = "%s"' % (
        table, myproperty, new_value, myproperty, old_value)
    LOG.debug(cmd)

    cursor.execute(cmd)
    conn.commit()
    conn.close()
    return

def get_granule_ids(short_name, product_ids, property_lookup):
    """
    return a list of dictionrys with the only element being the granule id
    """

    short_name_properties = []

    conn = sqlite3.connect(global_database_name)

    cursor = conn.cursor()

    prop = 'GRANULE_ID'
    table = property_lookup[prop]

    query = ('SELECT %s FROM %s WHERE %s LIKE "%s"  ' % (prop,
                                                         table, 'PRODUCT_ID',
                                                         product_ids[short_name]))

    LOG.debug(query)
    # cursor.execute(query)
    # srm ?all_rows = cursor.fetchall()
    for row in cursor.execute(query):
        gstr = row[0]
        gran_id_value = re.search('[a-zA-Z0-9_]+', gstr)
        gran_id = gstr[gran_id_value.start():gran_id_value.end()]

        info = {}
        info[prop] = gran_id
        info['PRODUCT_ID'] = str(product_ids[short_name])
        short_name_properties.append(info)

    conn.close()
    
    return short_name_properties


def get_granule_properties(granule_ids, property_lookup):
    """
    return a list of dictionrys with the only element being the granule id
    """
    conn = sqlite3.connect(global_database_name)
    cursor = conn.cursor()

    prop = 'GRANULE_ID'
    table = property_lookup[prop]

    for info in granule_ids:
        for prop in NEEDED_PROPS:

#             query = ('SELECT {0:s} FROM {1:s} WHERE {2:s} LIKE "{3:s}"  '.format(prop,
#                                                                                  table, 'GRANULE_ID',
#                                                                                  info['GRANULE_ID']))
            
            
            query = ('SELECT {0:s} FROM {1:s} WHERE {2:s} LIKE "{3:s}" AND {4:s} LIKE "{5:s}" '.format(prop,
                                                                                                       table, 'GRANULE_ID',
                                                                                                       info['GRANULE_ID'],
                                                                                                       'PRODUCT_ID',
                                                                                                       info['PRODUCT_ID']))



            LOG.debug(query)

            try:
                # LOG.debug('prop %s' % str(prop))
                for row in cursor.execute(query):
                    if row[0] is not None:
                        gstr = str(row)
                        prop_re = re.search('[a-zA-Z0-9_-]+', gstr)
                        prop_value = gstr[prop_re.start():prop_re.end()]
                        info[prop] = prop_value
                        LOG.debug('prop %s val %s' % (str(prop), prop_value))
                    else:
                        info[prop] = None
                        LOG.debug('set prop %s val %s' % (str(prop), info[prop]))

            except Exception:
                LOG.error(traceback.format_exc())
                pass

    conn.close()
    return granule_ids

from operator import itemgetter
def skim_dir(n_collection_short_name='SPACECRAFT-DIARY-RDR', spacecraft='NPP'):
    """yield a sequence of metadata dictionaries with optional keyword-content filtering
    optionally include iterable required_keys listing keys which must be available in the generated dictionaries
    this can also be accomplished by setting a matchkey as None, e.g. N_Granule_ID=None
    """
    LOG.debug('n_collection_short_name %s spacecraft %s'%(n_collection_short_name,spacecraft))
    short_name = n_collection_short_name
    # desired products ids and where the properties can be found
    product_ids, property_lookup = get_product_ids(spacecraft=spacecraft)
    LOG.debug(product_ids)
    LOG.debug(property_lookup)

    granule_ids = get_granule_ids(short_name, product_ids, property_lookup)
    if len(granule_ids) == 0: 
        LOG.warning('Granules not found, possible RDR problem. Check RDR acquisition software version %s %s'%(short_name,spacecraft))
        return None
    
    granule_ids2 = get_granule_properties(granule_ids, property_lookup)

    for gran in granule_ids2:
        for convert_me in NEEDED_PROPS_DT:
            #LOG.info(gran)

            if gran[convert_me] is not None:
                LOG.debug('convert %s %s ' % (str(convert_me), gran[convert_me]))
                granule_time = gran[convert_me]
                epoch = dt.datetime(1958, 1, 1)
                delta = dt.timedelta(microseconds=int(granule_time))
                gran[convert_me] = epoch + delta

    for gran in granule_ids2:
        #LOG.info(gran)
        for item in COMPAT:
            if item in gran:
                if COMPAT[item] in gran:  # already added
                    if gran[COMPAT[item]] is None:
                        LOG.debug('Replace/insert %s %s ' % (item, COMPAT[item]))
                        gran[COMPAT[item]] = gran[item]
 
                else:
                    # add it
                    LOG.debug('Add %s %s ' % (item, COMPAT[item]))
                    gran[COMPAT[item]] = gran[item]

    newlist = sorted(granule_ids2, key=itemgetter('N_Granule_ID'))

    last_list = list()
    b_gran = newlist[0]
    for gran in newlist[1:]:
        if b_gran['N_Granule_ID'] == gran['N_Granule_ID']:
            LOG.info('Duplicate found %s %s %s' % ( b_gran['N_Granule_ID'],gran['GRANULE_VERSION'],b_gran['GRANULE_VERSION']))
            if gran['GRANULE_VERSION']  < b_gran['GRANULE_VERSION'] :
                last_list.append( b_gran)      
        else :
            last_list.append( b_gran)     

        b_gran = gran
        
    last_list.append( b_gran) # add the last granule   

    LOG.debug(pp.pformat(last_list ))
    for info in last_list:
        yield info


def main():
    """ Run Viirs SDR proccessing
    """
    import argparse

    desc = """Build VIIRS SDR ProSdrViirsDBController work directory and run VIIRS SDR.
        viirs_sdr.sh wraps the python code, adl_viirs_sdr.py options are passed through."""
    parser = argparse.ArgumentParser(description=desc)

    parser.add_argument('-W', '--work-dir', metavar='work_dir', default='.',
                        help='directory in which all activity occurs, defaults to current dir')

    parser.add_argument('-d', '--debug',
                        action="store_true", default=False, help="always retain intermediate files")

    parser.add_argument('-v', '--verbosity', action="count", default=0,
                        help='each occurrence increases verbosity 1 level ERROR-WARNING-INFO-DEBUG')

    parser.add_argument("cspp_sdr_home", nargs='+', help='cspp_sdr_home')

    args = parser.parse_args()

    levels = [logging.ERROR, logging.WARN, logging.INFO, logging.DEBUG]
    level = levels[args.verbosity if args.verbosity < 4 else 3]

    work_dir = check_and_convert_path(args.work_dir,key="WORK_DIR")
    # d = dt.datetime.now()
    # timestamp = d.isoformat()
    # logname = "viirs_sdr." + timestamp + ".log"
    # logfile = os.path.join(work_dir, logname)
    # configure_logging(level, FILE=logfile)
    configure_logging(level)

    dpe_version = check_existing_env_var('DPE_VER', default_value='cspp',silent=True)
    LOG.info("product version: DPE_VER for VIIRS SDR  %s" % dpe_version)
    LOG.info('CSPP execution work directory is %r' % work_dir)

    if not args:
        parser.print_help()
        return 1

    rc = 1
    return rc


if __name__ == '__main__':
    sys.exit(main())
